"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var api_provider_1 = require("../../services/api/api.provider");
var Subject_1 = require("rxjs/Subject");
var account_1 = require("./account");
var transaction_1 = require("./transaction");
require("rxjs/add/operator/map");
var _ = require("lodash");
var AccountProvider = (function () {
    function AccountProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this.accounts = [];
        this.accountObservable = new Subject_1.Subject();
        this.accountsObservable = new Subject_1.Subject();
    }
    AccountProvider.prototype.accountState = function (result) {
        this.currentAccount = result;
        this.accountObservable.next(this.currentAccount);
        this.accountsObservable.next(this.accounts);
    };
    AccountProvider.prototype.create = function (userId) {
        var _this = this;
        var payload = {
            userId: userId
        };
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/account', payload)
                .then(function (response) {
                if (response.id) {
                    var account = account_1.Account.fromJson(response);
                    _this.accounts.push(account);
                    _this.accountState(account);
                    console.log('resolving account..');
                    resolve(account);
                }
                else {
                    reject('No id in account create response!');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.retrieveAllAccounts = function () {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            if (_this.accounts.length === 0) {
                _this.apiProvider.api('GET', '/v2/accounts', payload)
                    .then(function (response) {
                    if (response) {
                        _this.accounts = _this.deserializeAccounts(response);
                        _this.accountsObservable.next(_this.accounts);
                        console.log('resolving');
                        resolve(_this.accounts);
                    }
                    else {
                        reject();
                    }
                }, function (err) {
                    reject(err);
                });
            }
            else
                resolve(_this.accounts);
        });
        return promise;
    };
    AccountProvider.prototype.retrieveAccounts = function (filter) {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            if (_this.accounts.length === 0) {
                _this.apiProvider.api('GET', '/v2/accounts', filter)
                    .then(function (response) {
                    if (response) {
                        _this.accounts = _this.deserializeAccounts(response);
                        _this.accountsObservable.next(_this.accounts);
                        resolve(_this.accounts);
                    }
                    else {
                        reject();
                    }
                }, function (err) {
                    reject(err);
                });
            }
            else
                resolve(_this.accounts);
        });
        return promise;
    };
    AccountProvider.prototype.retrieve = function (accountId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/account/' + accountId, payload)
                .then(function (response) {
                if (response.id) {
                    var account_2 = account_1.Account.fromJson(response);
                    var position = _this.accounts.findIndex(function (item) { return item.id === account_2.id; });
                    _this.accounts.splice(position, 1, account_2);
                    _this.accountState(account_2);
                    resolve(account_2);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.update = function (userId, accountId) {
        var _this = this;
        var payload = {
            'userId': userId
        };
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', '/v1/account/' + accountId, payload)
                .then(function (response) {
                if (response.id) {
                    var account_3 = account_1.Account.fromJson(response);
                    var position = _this.accounts.findIndex(function (item) { return item.id === account_3.id; });
                    _this.accounts.splice(position, 1, account_3);
                    _this.accountState(account_3);
                    console.log('updateing account...');
                    resolve(account_3);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.delete = function (accountId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', '/v1/account/' + accountId, payload)
                .then(function (response) {
                if (response.id) {
                    var account_4 = account_1.Account.fromJson(response);
                    var position = _this.accounts.findIndex(function (item) { return item.id === account_4.id; });
                    _this.accounts.splice(position, 1);
                    _this.accountsObservable.next(_this.accounts);
                    console.log('deleting');
                    resolve(_this.accounts);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.changeStatus = function (accountId, newStatus) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.retrieve(accountId).then(function (result) {
                if (result.id) {
                    _this.apiProvider.api('PATCH', "/v1/account/" + accountId + "/status/" + newStatus, payload).then(function (response) {
                        if (response.id) {
                            var account_5 = account_1.Account.fromJson(response);
                            var position = _this.accounts.findIndex(function (item) { return item.id === account_5.id; });
                            _this.accounts.splice(position, 1, account_5);
                            _this.accountState(account_5);
                            console.log('resoving');
                            resolve(account_5);
                        }
                        else {
                            reject();
                        }
                    }, function (err) {
                        reject(err);
                    });
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.addTransaction = function (accountId, transaction) {
        var _this = this;
        var payload = transaction.toJson();
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/account/' + accountId, payload)
                .then(function (response) {
                if (response.id) {
                    var account_6 = account_1.Account.fromJson(response);
                    var position = _this.accounts.findIndex(function (item) { return item.id === account_6.id; });
                    _this.accounts.splice(position, 1, account_6);
                    _this.accountState(account_6);
                    resolve(account_6);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.retrieveTransaction = function (accountId, transactionId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/account/" + accountId + "/" + transactionId, payload)
                .then(function (response) {
                if (response.id) {
                    resolve(transaction_1.Transaction.fromJson(response));
                }
                else {
                    reject('Error retrieving accounts');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    AccountProvider.prototype.deserializeAccounts = function (incoming) {
        var accounts = _.map((_.values(incoming)), account_1.Account.fromJson);
        return accounts;
    };
    return AccountProvider;
}());
AccountProvider.decorators = [
    { type: core_1.Injectable },
];
AccountProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.AccountProvider = AccountProvider;
//# sourceMappingURL=account.provider.js.map