"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var http_1 = require("@angular/http");
require("rxjs/add/operator/map");
require("rxjs/add/operator/toPromise");
var ApiProvider = (function () {
    function ApiProvider(http) {
        this.http = http;
        this.appId = '571aff7c854c8dde5ea32fca';
        this.apiUri = '';
        this.socketUri = '';
        this.clientId = 'android';
        this.clientSecret = 'SomeRandomCharsAndNumbers';
        this.getToken = this.getOxlToken;
        this.prepareReq = this.prepareOxlReq;
        this.user = null;
        this.language = 'en-US';
    }
    ApiProvider.prototype.api = function (method, endpoint, payload) {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            _this.getToken().then(function (response) {
                if (!payload) {
                    payload = {};
                }
                var req = _this.prepareReq(method, endpoint, payload);
                if (method === 'GET' || method === 'DELETE' || method === 'PUT' || endpoint === '/v1/customer/checkin/') {
                    req.params = payload;
                }
                else {
                    req.data = payload;
                }
                _this.http.request(req).
                    map(function (res) { return res.json(); }).
                    toPromise().
                    then(function (responseN) {
                    if (responseN.errors) {
                        reject(responseN.errors);
                    }
                    else {
                        resolve(responseN.data);
                    }
                }, function (err) {
                    if (err.status || err.status == 0) {
                        reject(err.status);
                    }
                    else if (JSON.parse(err._body).errors) {
                        reject(JSON.parse(err._body).errors);
                    }
                    else {
                        reject(err);
                    }
                });
            });
        });
        return promise;
    };
    ApiProvider.prototype.prepareOxlReq = function (method, endpoint, payload) {
        var headers = new http_1.Headers();
        headers.append('Content-Type', 'application/json');
        if (this.accessToken) {
            headers.append('Authorization', 'Bearer ' + this.accessToken);
        }
        headers.append('Accept-Language', this.getLanguage());
        var params = new http_1.URLSearchParams();
        if (method === 'GET' || endpoint === '/v1/customer/checkin/') {
            for (var key in payload) {
                params.set(key, payload[key]);
            }
        }
        var basicOptions = {
            url: "" + this.apiUri + endpoint,
            method: method,
            search: params,
            headers: headers,
            body: payload
        };
        var reqOptions = new http_1.RequestOptions(basicOptions);
        return new http_1.Request(reqOptions);
    };
    ApiProvider.prototype.getOxlToken = function () {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            if (_this.accessToken) {
                resolve(_this.accessToken);
            }
            else {
                if (_this.user) {
                    var loginHeaders = new http_1.Headers();
                    loginHeaders.append('Content-Type', 'application/json');
                    _this.http.post(_this.apiUri + '/oauth/token', _this.user, { headers: loginHeaders }).
                        map(function (res) { return res.json(); }).
                        toPromise().
                        then(function (response) {
                        if (response.errors) {
                            reject('errors in oauth json');
                        }
                        else {
                            _this.accessToken = response.access_token;
                            _this.userId = response.userId;
                            _this.refreshToken = response.refresh_token;
                            resolve(_this.accessToken);
                        }
                    }, function (err) {
                        reject('Error in oauth' + err);
                    });
                }
                else {
                    _this.http.post(_this.apiUri + '/oauth/anonymous', {
                        client_id: _this.clientId,
                        client_secret: _this.clientSecret
                    }).
                        map(function (res) { return res.json(); }).
                        toPromise().
                        then(function (response) {
                        if (response.errors) {
                            reject('errors in oauth json');
                        }
                        else {
                            _this.accessToken = response.access_token;
                            resolve(_this.accessToken);
                        }
                    }, function (err) {
                        reject('Error in oauth' + err);
                    });
                }
            }
        });
        return promise;
    };
    ApiProvider.prototype.loginUser = function (payload) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.user = payload;
            var loginHeaders = new http_1.Headers();
            loginHeaders.append('Content-Type', 'application/json');
            _this.http.post(_this.apiUri + '/oauth/token', _this.user, { headers: loginHeaders }).
                map(function (res) { return res.json(); }).
                toPromise().
                then(function (response) {
                if (response.errors) {
                    reject('errors in oauth json');
                }
                else {
                    _this.accessToken = response.access_token;
                    _this.userId = response.userId;
                    _this.refreshToken = response.refresh_token;
                    console.log(response);
                    resolve(response);
                }
            }, function (err) {
                reject('Error in oauth' + err);
            });
        });
    };
    ApiProvider.prototype.logout = function () {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            _this.http.post(_this.apiUri + '/oauth/anonymous', {
                client_id: _this.clientId,
                client_secret: _this.clientSecret
            }).
                map(function (res) { return res.json(); }).
                toPromise().
                then(function (response) {
                if (response.errors) {
                    reject('errors in oauth json');
                }
                else {
                    _this.accessToken = response.access_token;
                    resolve(_this.accessToken);
                }
            }, function (err) {
                reject('Error in oauth' + err);
            });
        });
        return promise;
    };
    ApiProvider.prototype.resolver = function (resolve, reject) {
        setTimeout(function doneCounting() {
            resolve('testString');
        }, 2000);
    };
    ApiProvider.prototype.retrieveUserInfo = function () {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.api('GET', "/v1/user", payload)
                .then(function (response) {
                console.log(response);
                if (response.id) {
                    resolve(response);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    ApiProvider.prototype.setLanguage = function (lang) { this.language = lang; };
    ApiProvider.prototype.getLanguage = function () { return this.language; };
    ApiProvider.prototype.setApiUri = function (newUri) { this.apiUri = newUri; };
    ApiProvider.prototype.setSocketUri = function (newUri) { this.socketUri = newUri; };
    return ApiProvider;
}());
ApiProvider.decorators = [
    { type: core_1.Injectable },
];
ApiProvider.ctorParameters = function () { return [
    { type: http_1.Http, },
]; };
exports.ApiProvider = ApiProvider;
//# sourceMappingURL=api.provider.js.map