"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var api_provider_1 = require("../../services/api/api.provider");
var beacon_1 = require("./beacon");
require("rxjs/add/operator/map");
var Subject_1 = require("rxjs/Subject");
var _ = require("lodash");
var BeaconProvider = (function () {
    function BeaconProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this.beacons = [];
        this.beaconObservable = new Subject_1.Subject();
        this.beaconSetObservable = new Subject_1.Subject();
    }
    BeaconProvider.prototype.createBeacon = function (beacon) {
        var _this = this;
        var payload = beacon.toJson();
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/beacon', payload)
                .then(function (response) {
                if (response.id) {
                    _this.beaconState(beacon_1.Beacon.fromJson(response));
                    resolve(_this.currentBeacon);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    BeaconProvider.prototype.retrieveBeacon = function (beaconId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/beacon/' + beaconId, payload)
                .then(function (response) {
                if (response.id) {
                    _this.beaconState(beacon_1.Beacon.fromJson(response));
                    resolve(_this.currentBeacon);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    BeaconProvider.prototype.retrieveByTenant = function (tenantId) {
        var _this = this;
        var payload = {
            tenantId: tenantId
        };
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/beacons/', payload)
                .then(function (response) {
                if (response) {
                    _this.beacons = _.map((_.values(response)), beacon_1.Beacon.fromJson);
                    _this.beaconSetObservable.next(_this.beacons);
                    resolve(_this.beacons);
                }
                else {
                    reject();
                }
            })
                .catch(function (erresp) {
                reject(erresp);
            });
        });
        return promise;
    };
    BeaconProvider.prototype.createEvent = function (uuid, major, minor) {
        var _this = this;
        var payload = {};
        payload.uuid = uuid;
        payload.major = major;
        payload.minor = minor;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/beacon/event/presence', payload)
                .then(function (response) {
                if (response.id) {
                    _this.beaconState(beacon_1.Beacon.fromJson(response));
                    resolve(_this.currentBeacon);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    BeaconProvider.prototype.beaconState = function (result) {
        this.currentBeacon = result;
        this.beaconObservable.next(this.currentBeacon);
        this.beaconSetObservable.next(this.beacons);
    };
    return BeaconProvider;
}());
BeaconProvider.decorators = [
    { type: core_1.Injectable },
];
BeaconProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.BeaconProvider = BeaconProvider;
//# sourceMappingURL=beacon.provider.js.map