"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var moment = require("moment");
var Calendar = (function () {
    function Calendar(calendarId, tenantId, storeId, userId, title, createdOn, calEvents) {
        if (calEvents === void 0) { calEvents = []; }
        this.calendarId = calendarId;
        this.tenantId = tenantId;
        this.storeId = storeId;
        this.userId = userId;
        this.title = title;
        this.createdOn = createdOn;
        this.calEvents = calEvents;
    }
    Calendar.createDummyCalendar = function () {
        var dummyCalendar = new Calendar('__DUMMY__', '__DUMMY__', '__DUMMY__', '__DUMMY__', '__DUMMY__', moment('1970-01-01T00:00:00'), [Calendar.createDummyEvent()]);
        return dummyCalendar;
    };
    Calendar.createDummyEvent = function () {
        var dummyCalEvent = {
            id: '__DUMMY__',
            calendarId: '__DUMMY__',
            tenantId: '__DUMMY__',
            storeId: '__DUMMY__',
            userId: '__DUMMY__',
            title: '__DUMMY__',
            start: moment('1970-01-01T00:00:00'),
            end: moment('1970-01-01T00:00:00'),
            notes: '__DUMMY__',
            confirmedStatus: false,
            booking: {
                enabled: false,
                seated: false,
                touchpointId: '__DUMMY__',
                customers: null,
                customersNum: null,
                phone: null
            }
        };
        return dummyCalEvent;
    };
    Calendar.fromJson = function (obj) {
        var _this = this;
        var id = (obj.id) ? obj.id : '__FALLBACK__';
        var _tenantId = (obj.tenantId) ? obj.tenantId : '__FALLBACK__';
        var _storeId = (obj.storeId) ? obj.storeId : '__FALLBACK__';
        var _userId = (obj.userId) ? obj.userId : '__FALLBACK__';
        var _title = (obj.title) ? obj.title : '__FALLBACK__';
        var _createdOn = (obj.created) ? moment(obj.created) : moment('1970-01-01T00:00:00');
        var _events = [];
        if (obj.events) {
            _(obj.events).forEach(function (calEvent) {
                _events.push(_this.eventFromJson(calEvent));
            });
        }
        return new Calendar(id, _tenantId, _storeId, _userId, _title, _createdOn, _events);
    };
    Calendar.bookingFromJson = function (obj) {
        var _touchpointId = (obj.touchpointId) ? obj.touchpointId : '__FALLBACK__';
        var _customers = (obj.customers) ? obj.customers : ['Guest'];
        var _enabled = (obj.enabled) ? obj.enabled : false;
        var _seated = (obj.seated) ? obj.seated : false;
        var _customersNum = (obj.customersNum) ? obj.customersNum : 1;
        var _phone = (obj.phone) ? obj.phone : 1337;
        var newBooking = {
            enabled: _enabled,
            seated: _seated,
            touchpointId: _touchpointId,
            customers: _customers,
            customersNum: _customersNum,
            phone: _phone
        };
        return newBooking;
    };
    Calendar.eventFromJson = function (obj) {
        var _eventId = (obj.id) ? obj.id : '__FALLBACK__';
        var _calendarId = (obj.calendarId) ? obj.calendarId : '__FALLBACK__';
        var _tenantId = (obj.tenantId) ? obj.tenantId : '__FALLBACK__';
        var _storeId = (obj.storeId) ? obj.storeId : '__FALLBACK__';
        var _userId = (obj.userId) ? obj.userId : '__FALLBACK__';
        var _title = (obj.title) ? obj.title : '__FALLBACK__';
        var _start = (obj.startDate) ? moment(obj.startDate) : moment('1970-01-01T00:00:00');
        var _end = (obj.endDate) ? moment(obj.endDate) : moment('1970-01-01T00:00:00');
        var _notes = (obj.notes) ? obj.notes : '__FALLBACK__';
        var _confirmed = (obj.confirmed) ? obj.confirmed : false;
        var _booking = (obj.booking) ? this.bookingFromJson(obj.booking) : {
            enabled: false,
            seated: false,
            touchpointId: null,
            customers: null,
            customersNum: null,
            phone: null
        };
        var newCalEvent = {
            id: _eventId,
            calendarId: _calendarId,
            tenantId: _tenantId,
            storeId: _storeId,
            userId: _userId,
            title: _title,
            start: _start,
            end: _end,
            notes: _notes,
            confirmedStatus: _confirmed,
            booking: _booking
        };
        return newCalEvent;
    };
    Calendar.updateOrPushCalendar = function (calendarToUpdate, calendars) {
        var indexOfItemToUpdate = _.findIndex(calendars, function (cal) {
            return cal.getCalendarId() === calendarToUpdate.getCalendarId();
        });
        if (indexOfItemToUpdate !== -1) {
            calendars[indexOfItemToUpdate] = calendarToUpdate;
        }
        else if (indexOfItemToUpdate === -1) {
            calendars.push(calendarToUpdate);
            if (!(calendars.length === 1)) {
                console.warn('Warning: Former version of updated calendar was not found in local cache, but cache was synced anyway.');
            }
        }
    };
    Calendar.deleteCalendar = function (calendarToDelete, calendars) {
        var indexOfItemToDelete = _.findIndex(calendars, function (cal) {
            return cal.getCalendarId() === calendarToDelete.getCalendarId();
        });
        if (indexOfItemToDelete !== -1) {
            calendars.splice(indexOfItemToDelete, 1);
            if (calendars.length === 0) {
                calendars.push(Calendar.createDummyCalendar());
            }
        }
        else if (indexOfItemToDelete === -1) {
            console.warn('Warning: Former version of deleted calendar not found in local cache.');
        }
    };
    Calendar.updateOrPushEvent = function (eventToUpdate, events) {
        if (events[0].id === '__DUMMY__') {
            events.splice(0);
        }
        var deletedEvents = _.remove(events, function (ev) {
            return ev.id === eventToUpdate.id;
        });
        events.push(eventToUpdate);
        if (deletedEvents.length === 0) {
            console.warn('Warning: Former version of updated event was not found in local cache, but cache was synced anyway.');
        }
    };
    Calendar.deleteEvent = function (eventToDelete, events) {
        var deletedEvents = _.remove(events, function (e) {
            return e.id === eventToDelete.id;
        });
        if (deletedEvents.length <= 0) {
            console.warn("Warning: Deleting event from local cache failed because no former version of the deleted event was found.");
        }
    };
    Calendar.prototype.findEvent = function (eventId) {
        return _.find(this.getCalEvents(), function (e) {
            return e.id === eventId;
        });
    };
    Calendar.prototype.getCalendarId = function () { return this.calendarId; };
    Calendar.prototype.getTenantId = function () { return this.tenantId; };
    Calendar.prototype.getStoreId = function () { return this.storeId; };
    Calendar.prototype.getUserId = function () { return this.userId; };
    Calendar.prototype.getTitle = function () { return this.title; };
    Calendar.prototype.getCreatedOn = function () { return this.createdOn; };
    Calendar.prototype.getCalEvents = function () { return this.calEvents; };
    return Calendar;
}());
exports.Calendar = Calendar;
//# sourceMappingURL=calendar.js.map