/// <reference types="moment" />
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import 'rxjs/add/operator/map';
import { Calendar, CalEvent, Booking } from './calendar';
import { ApiProvider } from '../api/api.provider';
import * as moment from 'moment';
export declare class CalendarProvider {
    apiProvider: ApiProvider;
    _calendar$: Subject<Calendar>;
    _calendars$: Subject<Calendar[]>;
    _upcomingEvents$: Subject<CalEvent[]>;
    _currUpcomingEvent$: Subject<CalEvent>;
    dataStore: {
        calendar: Calendar;
        calendars: Calendar[];
        upcomingEvents: CalEvent[];
        start: moment.Moment;
        end: moment.Moment;
    };
    readonly calendar$: Observable<Calendar>;
    readonly calendars$: Observable<Calendar[]>;
    readonly upcomingEvents$: Observable<CalEvent[]>;
    readonly currentUpcomingEvent$: Observable<CalEvent>;
    constructor(apiProvider: ApiProvider);
    initCalendar(tenantId: string, storeId: string): Promise<Calendar>;
    createCalendar(title: string, tenantId: string, storeId: string): Promise<Calendar>;
    retrieveCalendarDetails(calendarId: string, tenantId: string, storeId: string): Promise<Calendar>;
    retrieveCalendars(tenantId: string, storeId: string): Promise<Calendar[]>;
    updateCalendar(calendarId: string, tenantId: string, storeId: string, title: string): Promise<Calendar>;
    deleteCalendar(calendarId: string, tenantId: string, storeId: string, title: string): Promise<Calendar>;
    createEvent(calendarId: string, tenantId: string, storeId: string, title: string, start: moment.Moment, end: moment.Moment, notes: string, booking: Booking): Promise<CalEvent>;
    retrieveEvent(calEventId: string, calendarId: string): Promise<CalEvent>;
    updateEvent(calEventId: string, calendarId: string, tenantId: string, storeId: string, title: string, start: moment.Moment, end: moment.Moment, notes: string, booking: Booking): Promise<CalEvent>;
    deleteEvent(calEventId: string, calendarId: string): Promise<CalEvent>;
    retrieveUpcomingEvents(calendarId: string, start: moment.Moment, end: moment.Moment): Promise<CalEvent[]>;
    updateEventStatus(calEventId: string, calendarId: string, confirmStatus: boolean): Promise<CalEvent>;
}
