"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var Subject_1 = require("rxjs/Subject");
require("rxjs/add/operator/map");
var calendar_1 = require("./calendar");
var api_provider_1 = require("../api/api.provider");
var _ = require("lodash");
var CalendarProvider = (function () {
    function CalendarProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this._calendar$ = new Subject_1.Subject();
        this._calendars$ = new Subject_1.Subject();
        this._upcomingEvents$ = new Subject_1.Subject();
        this._currUpcomingEvent$ = new Subject_1.Subject();
        var dummyCal = calendar_1.Calendar.createDummyCalendar();
        var dummyEvent = calendar_1.Calendar.createDummyEvent();
        this.dataStore = {
            calendar: dummyCal,
            calendars: [dummyCal],
            upcomingEvents: [dummyEvent],
            start: undefined,
            end: undefined
        };
    }
    Object.defineProperty(CalendarProvider.prototype, "calendar$", {
        get: function () { return this._calendar$.asObservable(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CalendarProvider.prototype, "calendars$", {
        get: function () { return this._calendars$.asObservable(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CalendarProvider.prototype, "upcomingEvents$", {
        get: function () { return this._upcomingEvents$.asObservable(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CalendarProvider.prototype, "currentUpcomingEvent$", {
        get: function () { return this._currUpcomingEvent$.asObservable(); },
        enumerable: true,
        configurable: true
    });
    CalendarProvider.prototype.initCalendar = function (tenantId, storeId) {
        var _this = this;
        if (this.dataStore.calendar.getCalendarId() !== '__DUMMY__') {
            return new Promise(function (resolve, reject) { resolve(_this.dataStore.calendar); });
        }
        else {
            return new Promise(function (resolve, reject) {
                _this.retrieveCalendars(tenantId, storeId)
                    .then(function (cals) {
                    if (cals.length === 0) {
                        _this.createCalendar('DEFAULT_CALENDAR_NAME', tenantId, storeId)
                            .then(function (cal) { resolve(cal); });
                    }
                    else {
                        _this.retrieveCalendarDetails(_.find(cals, function (o) { return o.tenantId === tenantId; }).getCalendarId(), tenantId, storeId)
                            .then(function (cal) { resolve(cal); });
                    }
                })
                    .catch(function (error) {
                    console.error("Error: initCalendar() failed with response: error " + error);
                    reject(error);
                });
            });
        }
    };
    CalendarProvider.prototype.createCalendar = function (title, tenantId, storeId) {
        var _this = this;
        var payload = {
            'title': title,
            'tenantId': tenantId,
            'storeId': storeId
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/calendar', payload)
                .then(function (response) {
                var createdCalendar = calendar_1.Calendar.fromJson(response);
                resolve(createdCalendar);
                _this.dataStore.calendar = createdCalendar;
                _this._calendar$.next(_this.dataStore.calendar);
            })
                .catch(function (error) {
                console.error("Error: createCalendar() API request failed with response: error " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.retrieveCalendarDetails = function (calendarId, tenantId, storeId) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/calendar/" + calendarId, payload)
                .then(function (response) {
                var retrievedCalendar = calendar_1.Calendar.fromJson(response);
                resolve(retrievedCalendar);
                _this.dataStore.calendar = retrievedCalendar;
                calendar_1.Calendar.updateOrPushCalendar(retrievedCalendar, _this.dataStore.calendars);
                _this._calendar$.next(_this.dataStore.calendar);
                _this._calendars$.next(_this.dataStore.calendars);
            })
                .catch(function (error) {
                console.error("Error: retrieveCalendarDetails() API request failed with response: error " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.retrieveCalendars = function (tenantId, storeId) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/calendars', payload)
                .then(function (response) {
                if (response.length > 0) {
                    _this.dataStore.calendars.splice(0);
                    _(response).forEach(function (cal) {
                        calendar_1.Calendar.updateOrPushCalendar(calendar_1.Calendar.fromJson(cal), _this.dataStore.calendars);
                    });
                    resolve(_this.dataStore.calendars);
                    _this._calendars$.next(_this.dataStore.calendars);
                }
            })
                .catch(function (error) {
                if (error === 700) {
                    resolve([]);
                }
                else {
                    console.error("Error: retrieveCalendars() API request failed with response: error " + error + ",  " + error);
                    reject(error);
                }
            });
        });
    };
    CalendarProvider.prototype.updateCalendar = function (calendarId, tenantId, storeId, title) {
        var _this = this;
        var payload = {
            'title': title,
            'tenantId': tenantId,
            'storeId': storeId
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/calendar/" + calendarId, payload)
                .then(function (response) {
                var updatedCalendar = calendar_1.Calendar.fromJson(response);
                resolve(updatedCalendar);
                _this.dataStore.calendar = updatedCalendar;
                calendar_1.Calendar.updateOrPushCalendar(updatedCalendar, _this.dataStore.calendars);
                _this._calendar$.next(_this.dataStore.calendar);
                _this._calendars$.next(_this.dataStore.calendars);
            })
                .catch(function (error) {
                console.error("Error: updateCalendar() API request failed with: error " + error + ",  " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.deleteCalendar = function (calendarId, tenantId, storeId, title) {
        var _this = this;
        var payload = {
            'title': title,
            'tenantId': tenantId,
            'storeId': storeId
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/calendar/" + calendarId, payload)
                .then(function (response) {
                var deletedCalendar = calendar_1.Calendar.fromJson(response);
                resolve(deletedCalendar);
                _this.dataStore.calendar = calendar_1.Calendar.createDummyCalendar();
                calendar_1.Calendar.deleteCalendar(deletedCalendar, _this.dataStore.calendars);
                _this._calendar$.next(_this.dataStore.calendar);
                _this._calendars$.next(_this.dataStore.calendars);
            })
                .catch(function (error) {
                console.error("Error: deleteCalendar() API request failed with: error " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.createEvent = function (calendarId, tenantId, storeId, title, start, end, notes, booking) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId,
            'title': title,
            'startDate': start,
            'endDate': end,
            'notes': notes,
            'booking': booking
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', "/v1/calendar/" + calendarId, payload)
                .then(function (response) {
                var newCalEvent = calendar_1.Calendar.eventFromJson(response);
                resolve(newCalEvent);
                calendar_1.Calendar.updateOrPushEvent(newCalEvent, _this.dataStore.calendar.getCalEvents());
                _this._calendar$.next(_this.dataStore.calendar);
                if (_this.dataStore.start !== undefined) {
                    if ((_this.dataStore.start.diff(newCalEvent.start, 'seconds') <= 0)
                        && (_this.dataStore.end.diff(newCalEvent.start, 'seconds') >= 0)) {
                        calendar_1.Calendar.updateOrPushEvent(newCalEvent, _this.dataStore.upcomingEvents);
                        _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                        _this._currUpcomingEvent$.next(newCalEvent);
                    }
                }
            })
                .catch(function (error) {
                console.error("Error: createCalEvent() API request failed with response: error " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.retrieveEvent = function (calEventId, calendarId) {
        var _this = this;
        var payload = {};
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/calendar/" + calendarId + "/" + calEventId, payload)
                .then(function (response) {
                var retrievedCalEvent = calendar_1.Calendar.eventFromJson(response);
                resolve(retrievedCalEvent);
                calendar_1.Calendar.updateOrPushEvent(retrievedCalEvent, _this.dataStore.calendar.getCalEvents());
                _this._calendar$.next(_this.dataStore.calendar);
                if (_this.dataStore.start !== undefined) {
                    if ((_this.dataStore.start.diff(retrievedCalEvent.start, 'seconds') <= 0)
                        && (_this.dataStore.end.diff(retrievedCalEvent.start, 'seconds') >= 0)) {
                        calendar_1.Calendar.updateOrPushEvent(retrievedCalEvent, _this.dataStore.upcomingEvents);
                        _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                        _this._currUpcomingEvent$.next(retrievedCalEvent);
                    }
                }
            })
                .catch(function (error) {
                console.error("Error: retrieveCalEvent() API request failed with: error " + error + ",  " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.updateEvent = function (calEventId, calendarId, tenantId, storeId, title, start, end, notes, booking) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId,
            'title': title,
            'startDate': start,
            'endDate': end,
            'notes': notes,
            'booking': booking
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/calendar/" + calendarId + "/" + calEventId, payload)
                .then(function (response) {
                var updatedCalEvent = calendar_1.Calendar.eventFromJson(response);
                resolve(updatedCalEvent);
                calendar_1.Calendar.updateOrPushEvent(updatedCalEvent, _this.dataStore.calendar.getCalEvents());
                _this._calendar$.next(_this.dataStore.calendar);
                if (_this.dataStore.start !== undefined) {
                    if ((_this.dataStore.start.diff(updatedCalEvent.start, 'seconds') <= 0)
                        && (_this.dataStore.end.diff(updatedCalEvent.end, 'seconds') >= 0)) {
                        calendar_1.Calendar.updateOrPushEvent(updatedCalEvent, _this.dataStore.upcomingEvents);
                        _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                        _this._currUpcomingEvent$.next(updatedCalEvent);
                    }
                    else {
                        calendar_1.Calendar.deleteEvent(updatedCalEvent, _this.dataStore.upcomingEvents);
                        _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                        _this._currUpcomingEvent$.next(updatedCalEvent);
                    }
                }
            })
                .catch(function (error) {
                console.error("Error: updateCalEvent() API request failed with: error " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.deleteEvent = function (calEventId, calendarId) {
        var _this = this;
        var payload = {};
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/calendar/" + calendarId + "/" + calEventId, payload)
                .then(function (response) {
                var deletedCalEvent = calendar_1.Calendar.eventFromJson(response);
                resolve(deletedCalEvent);
                calendar_1.Calendar.deleteEvent(deletedCalEvent, _this.dataStore.calendar.getCalEvents());
                calendar_1.Calendar.deleteEvent(deletedCalEvent, _this.dataStore.upcomingEvents);
                _this._calendar$.next(_this.dataStore.calendar);
                _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                _this._currUpcomingEvent$.next(deletedCalEvent);
            })
                .catch(function (error) {
                console.error("Error: deleteCalEvent() API request failed with response: error " + error + ",  " + error);
                reject(error);
            });
        });
    };
    CalendarProvider.prototype.retrieveUpcomingEvents = function (calendarId, start, end) {
        var _this = this;
        this.dataStore.start = start;
        this.dataStore.end = end;
        var payload = {
            'startDate': start,
            'endDate': end
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', "/v1/calendar/" + calendarId + "/upcoming", payload)
                .then(function (response) {
                if (response.length > 0) {
                    _this.dataStore.upcomingEvents.splice(0);
                    var retrievedUpcomingCalEvents_1 = [];
                    _(response).forEach(function (event) {
                        retrievedUpcomingCalEvents_1.push(calendar_1.Calendar.eventFromJson(event));
                    });
                    resolve(retrievedUpcomingCalEvents_1);
                    _this.dataStore.upcomingEvents = retrievedUpcomingCalEvents_1;
                    _(retrievedUpcomingCalEvents_1).forEach(function (event) {
                        calendar_1.Calendar.updateOrPushEvent(calendar_1.Calendar.eventFromJson(event), _this.dataStore.calendar.getCalEvents());
                    });
                    _this._calendar$.next(_this.dataStore.calendar);
                    _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                }
            })
                .catch(function (error) {
                if (error === 700) {
                    resolve([]);
                }
                else {
                    console.error("Error: retrieveUpcomingEvents() API request failed with response: error " + error + ".");
                    reject(error);
                }
            });
        });
    };
    CalendarProvider.prototype.updateEventStatus = function (calEventId, calendarId, confirmStatus) {
        var _this = this;
        var payload = { 'confirmed': String(confirmStatus) };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/calendar/" + calendarId + "/" + calEventId + "/confirm", payload)
                .then(function (response) {
                var updatedCalEvent = calendar_1.Calendar.eventFromJson(response);
                resolve(updatedCalEvent);
                calendar_1.Calendar.updateOrPushEvent(updatedCalEvent, _this.dataStore.calendar.getCalEvents());
                _this._calendar$.next(_this.dataStore.calendar);
                if (_this.dataStore.start !== undefined) {
                    if ((_this.dataStore.start.diff(updatedCalEvent.start, 'seconds') <= 0)
                        && (_this.dataStore.end.diff(updatedCalEvent.start, 'seconds') >= 0)) {
                        calendar_1.Calendar.updateOrPushEvent(updatedCalEvent, _this.dataStore.upcomingEvents);
                        _this._upcomingEvents$.next(_this.dataStore.upcomingEvents);
                        _this._currUpcomingEvent$.next(updatedCalEvent);
                    }
                }
            })
                .catch(function (error) {
                console.error("Error: updateEventStatus() API request failed with response: error " + error + ",  " + error);
                reject(error);
            });
        });
    };
    return CalendarProvider;
}());
CalendarProvider.decorators = [
    { type: core_1.Injectable },
];
CalendarProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.CalendarProvider = CalendarProvider;
//# sourceMappingURL=calendar.provider.js.map