"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var api_provider_1 = require("../../services/api/api.provider");
var Subject_1 = require("rxjs/Subject");
var cart_1 = require("./cart");
var product_1 = require("../product/product");
require("rxjs/add/operator/map");
var _ = require("lodash");
var CartsProvider = (function () {
    function CartsProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this.carts = [];
        this.currentCart = cart_1.Cart.fromJson(this.emptyCart());
        this.cartObservable = new Subject_1.Subject();
        this.cartsObservable = new Subject_1.Subject();
    }
    CartsProvider.prototype.cartState = function (result) {
        this.currentCart = result;
        this.cartObservable.next(this.currentCart);
        this.cartsObservable.next(this.carts);
    };
    CartsProvider.prototype.emptyCart = function (tenantId, storeId) {
        var empty;
        empty = [];
        return {
            '__v': '',
            'userId': '',
            'storeId': storeId,
            'tenantId': tenantId,
            'id': '',
            'created': '',
            'status': '',
            'items': empty,
            'numItems': 0,
            'total': 0
        };
    };
    CartsProvider.prototype.create = function (tenantId, storeId, userId) {
        var _this = this;
        var payload = {};
        payload.storeId = storeId;
        payload.tenantId = tenantId;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/cart', payload)
                .then(function (response) {
                if (response.id) {
                    _this.update(tenantId, storeId, userId, response.id).then(function (cart) { return resolve(cart); });
                }
                else {
                    reject('No id in cart create response!');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.retrieve = function (cartId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/cart/' + cartId, payload)
                .then(function (response) {
                if (response.id) {
                    var cart_2 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_2.id; });
                    _this.carts.splice(position, 1, cart_2);
                    _this.cartState(cart_2);
                    resolve(cart_2);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.retrieveAllCarts = function (tenantId, storeId) {
        var _this = this;
        var payload = {};
        if (tenantId)
            payload.tenantId = tenantId;
        if (storeId)
            payload.storeId = storeId;
        var promise = new Promise(function (resolve, reject) {
            if (_this.carts.length === 0) {
                _this.apiProvider.api('GET', '/v1/carts', payload)
                    .then(function (response) {
                    if (response) {
                        _this.carts = _this.deserializeCarts(response);
                        _this.cartsObservable.next(_this.carts);
                        console.log('resolving');
                        resolve(_this.carts);
                    }
                    else {
                        reject();
                    }
                }, function (err) {
                    reject(err);
                });
            }
            else
                resolve(_this.carts);
        });
        return promise;
    };
    CartsProvider.prototype.retrieveByUser = function (tenantId, userId) {
        var _this = this;
        var promise = new Promise(function (resolve) {
            resolve(_this.carts.filter(function (cart) { return (cart.tenantId === tenantId) && (cart.userId === userId); }));
        });
        return promise;
    };
    CartsProvider.prototype.retrieveByUserAndStore = function (tenantId, storeId, userId) {
        var _this = this;
        var promise = new Promise(function (resolve) {
            resolve(_this.carts.filter(function (cart) { return (cart.tenantId === tenantId) && (cart.userId === userId) && (cart.storeId === storeId); }));
        });
        return promise;
    };
    CartsProvider.prototype.addItem = function (cartId, product) {
        var _this = this;
        var payload = product.toJson();
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/cart/' + cartId, payload)
                .then(function (response) {
                if (response.id) {
                    var cart_3 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_3.id; });
                    _this.carts.splice(position, 1, cart_3);
                    _this.cartState(cart_3);
                    resolve(cart_3);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.update = function (tenantId, storeId, userId, cartId, tableId) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId,
            'userId': userId
        };
        if (tableId)
            payload.tableId = tableId;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', '/v1/cart/' + cartId, payload)
                .then(function (response) {
                if (response.id) {
                    var cart_4 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_4.id; });
                    if (position > -1) {
                        _this.carts.splice(position, 1, cart_4);
                    }
                    else {
                        _this.carts.push(cart_4);
                    }
                    _this.cartState(cart_4);
                    console.log('deleting');
                    resolve(cart_4);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.delete = function (cartId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', '/v1/cart/' + cartId, payload)
                .then(function (response) {
                if (response.id) {
                    var cart_5 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_5.id; });
                    _this.carts.splice(position, 1);
                    _this.cartsObservable.next(_this.carts);
                    console.log('deleting');
                    resolve(_this.carts);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.changeStatus = function (cartId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.retrieve(cartId).then(function (result) {
                if (result.id) {
                    var newStatus = (_this.currentCart.status === 'open' ? 'closed' : 'open');
                    _this.apiProvider.api('PATCH', "/v1/cart/" + cartId + "/status/" + newStatus, payload).then(function (response) {
                        if (response.id) {
                            var cart_6 = cart_1.Cart.fromJson(response);
                            var position = _this.carts.findIndex(function (item) { return item.id === cart_6.id; });
                            _this.carts.splice(position, 1, cart_6);
                            _this.cartState(cart_6);
                            console.log('resoving');
                            resolve(cart_6);
                        }
                        else {
                            reject();
                        }
                    }, function (err) {
                        reject(err);
                    });
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.paid = function (cartId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', "/v1/cart/" + cartId + "/paid", payload)
                .then(function (response) {
                if (response.id) {
                    var cart_7 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_7.id; });
                    _this.carts.splice(position, 1, cart_7);
                    _this.cartState(cart_7);
                    console.log('resolving');
                    resolve(cart_7);
                }
                else {
                    reject('Error cart#1');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.retrieveItem = function (cartId, productId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/cart/" + cartId + "/" + productId, payload)
                .then(function (response) {
                if (response.productId) {
                    resolve(product_1.Product.inCartFromJson(response));
                }
                else {
                    reject('Error retrieving carts');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.updateCartItem = function (cartId, productId, updatedProduct) {
        var _this = this;
        var payload = updatedProduct.toJson();
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/cart/" + cartId + "/" + productId, payload)
                .then(function (response) {
                if (response.id) {
                    var cart_8 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_8.id; });
                    _this.carts.splice(position, 1, cart_8);
                    _this.cartState(cart_8);
                    console.log('updating');
                    resolve(cart_8);
                }
                else {
                    reject('Error in updating Cart Item');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.deleteItem = function (cartId, productId) {
        var _this = this;
        console.log("deleting from cart " + cartId + " the following Item " + productId);
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/cart/" + cartId + "/" + productId, payload)
                .then(function (response) {
                if (response.id) {
                    var cart_9 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_9.id; });
                    _this.carts.splice(position, 1, cart_9);
                    _this.cartState(cart_9);
                    console.log('deleting item');
                    resolve(cart_9);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.deleteAllItems = function (cartId) {
        var _this = this;
        console.log("deleting all items from cart " + cartId + " ");
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/cart/" + cartId + "/flush", payload)
                .then(function (response) {
                if (response.id) {
                    var cart_10 = cart_1.Cart.fromJson(response);
                    var position = _this.carts.findIndex(function (item) { return item.id === cart_10.id; });
                    _this.carts.splice(position, 1, cart_10);
                    _this.cartState(cart_10);
                    console.log('deleting item');
                    resolve(cart_10);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    CartsProvider.prototype.deserializeCarts = function (incoming) {
        var carts = _.map((_.values(incoming)), cart_1.Cart.fromJson);
        return carts;
    };
    return CartsProvider;
}());
CartsProvider.decorators = [
    { type: core_1.Injectable },
];
CartsProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.CartsProvider = CartsProvider;
//# sourceMappingURL=cart.provider.js.map