"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var http_1 = require("@angular/http");
require("rxjs/add/operator/map");
var api_provider_1 = require("../api/api.provider");
var Subject_1 = require("rxjs/Subject");
var _ = require("lodash");
var client_1 = require("./client");
var ClientProvider = (function () {
    function ClientProvider(http, api) {
        this.http = http;
        this.api = api;
        this.isManager = false;
        this.clients = [];
        this.clientsObservable = new Subject_1.Subject();
        this.instore = [];
        this.instoreObservable = new Subject_1.Subject();
        this.clientObservable = new Subject_1.Subject();
    }
    ClientProvider.prototype.setManagerStatus = function (rank) {
        this.isManager = rank;
    };
    ClientProvider.prototype.getManagerStatus = function () {
        return this.isManager;
    };
    ClientProvider.prototype.getCustomersInStore = function (tenantId, storeId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            if (tenantId) {
                _this.api.api('GET', "/v1/customers/instore/" + storeId, payload)
                    .then(function (response) {
                    resolve(_this.deserializeCustomers(response));
                }, function (response) {
                    if (response === 400) {
                        resolve([]);
                    }
                    else
                        reject(response);
                });
            }
            else {
                reject();
            }
        });
        return promise;
    };
    ClientProvider.prototype.getCustomers = function (tenantId, storeId) {
        var _this = this;
        var payload;
        if (storeId) {
            payload = {
                storeId: storeId
            };
        }
        var promise = new Promise(function (resolve, reject) {
            if (tenantId) {
                payload.tenantId = tenantId;
                _this.api.api('GET', '/v1/customers/', payload)
                    .then(function (response) {
                    resolve(_this.deserializeCustomers(response));
                }, function (response) {
                    reject(response);
                });
            }
            else {
                reject();
            }
        });
        return promise;
    };
    ClientProvider.prototype.loadClients = function (tenantId, storeId) {
        var _this = this;
        return new Promise(function (resolve) {
            if (_this.clients.length === 0) {
                _this.getCustomers(tenantId, storeId).then(function (result) {
                    _this.clients = result;
                    _this.clientsObservable.next(_this.clients);
                    resolve(_this.clients);
                });
            }
            else {
                _this.clientsObservable.next(_this.clients);
                resolve(_this.clients);
            }
        });
    };
    ClientProvider.prototype.loadClientsInStore = function (tenantId, storeId) {
        var _this = this;
        return new Promise(function (resolve) {
            _this.getCustomersInStore(tenantId, storeId).then(function (result) {
                _this.instore = result;
                _this.instoreObservable.next(_this.instore);
                resolve(_this.instore);
            });
        });
    };
    ClientProvider.prototype.getCustomersDetails = function (tenantId, storeId, userId, customerId) {
        var _this = this;
        var payload = {};
        if (storeId) {
            payload = {
                storeId: storeId,
                userId: userId
            };
        }
        if (_.isUndefined(customerId)) {
            customerId = '';
        }
        var promise = new Promise(function (resolve, reject) {
            if (tenantId) {
                payload.tenantId = tenantId;
                _this.api.api('GET', "/v1/customer/" + customerId, payload)
                    .then(function (response) {
                    var customer = client_1.Customer.fromJson(response);
                    var position = _this.clients.findIndex(function (item) { return item.id === customer.id; });
                    _this.clients.splice(position, 1, customer);
                    _this.clientObservable.next(customer);
                    _this.clientsObservable.next(_this.clients);
                    resolve(client_1.Customer.fromJson(response));
                }, function (response) {
                    reject(response);
                });
            }
            else {
                reject();
            }
        });
        return promise;
    };
    ClientProvider.prototype.createCustomer = function (tenantId, storeId, fname, lname, social, addresses, telephones, badges) {
        var _this = this;
        var payload;
        if (storeId) {
            payload = {
                storeId: storeId
            };
        }
        payload.firstName = fname;
        payload.lastName = lname;
        payload.social = social;
        payload.addresses = addresses;
        payload.telephones = telephones;
        payload.badges = badges;
        var promise = new Promise(function (resolve, reject) {
            if (tenantId) {
                payload.tenantId = tenantId;
                _this.api.api('POST', '/v1/customer', payload)
                    .then(function (response) {
                    var customer = client_1.Customer.fromJson(response);
                    _this.clients.push(customer);
                    _this.clientsObservable.next(_this.clients);
                    _this.clientObservable.next(customer);
                    resolve(customer);
                }, function (response) {
                    reject(response);
                });
            }
            else {
                reject();
            }
        });
        return promise;
    };
    ClientProvider.prototype.updateCustomer = function (customerUpdate) {
        var _this = this;
        var payload = customerUpdate.toJson();
        return new Promise(function (resolve, reject) {
            _this.api.api('PUT', "/v1/customer/" + customerUpdate.getId(), payload)
                .then(function (response) {
                var customer = client_1.Customer.fromJson(response);
                var position = _this.clients.findIndex(function (item) { return item.id === customer.id; });
                _this.clients.splice(position, 1, customer);
                _this.clientsObservable.next(_this.clients);
                _this.clientObservable.next(customer);
                resolve(customer);
            }, function (response) {
                reject(response);
            });
        });
    };
    ClientProvider.prototype.deserializeCustomers = function (incoming) {
        var customers = [];
        for (var att in incoming) {
            var customer = incoming[att];
            customers.push(new client_1.Customer(customer.tenantId, customer.firstName, customer.lastName, customer.avatar, customer.gender, customer.id, customer.social, customer.badges, customer.addresses, customer.telephones, customer.storeId, customer.userId));
        }
        return customers;
    };
    ClientProvider.prototype.testCustomer = function () {
        console.log(this.deserializeCustomers(this.clients));
    };
    ClientProvider.prototype.checkIn = function (tenantId, storeId, userId, inStore, customerId) {
        var _this = this;
        var payload;
        if (storeId) {
            payload = {
                storeId: storeId,
                inStore: inStore,
                userId: userId,
            };
        }
        if (_.isUndefined(customerId)) {
            customerId = '';
        }
        var promise = new Promise(function (resolve, reject) {
            if (tenantId) {
                payload.tenantId = tenantId;
                _this.api.api('POST', "/v1/customer/checkin/" + customerId, payload)
                    .then(function (response) {
                    var customer = client_1.Customer.fromJson(response);
                    var position = _this.clients.findIndex(function (item) { return item.id === customer.id; });
                    _this.clients.splice(position, 1, customer);
                    _this.clientObservable.next(customer);
                    _this.clientsObservable.next(_this.clients);
                    resolve(client_1.Customer.fromJson(response));
                }, function (response) {
                    reject(response);
                });
            }
            else {
                reject();
            }
        });
        return promise;
    };
    ClientProvider.prototype.engaged = function (tenantId, storeId, userId, touchpointId, inStore, customerId) {
        var _this = this;
        var payload;
        if (storeId) {
            payload = {
                storeId: storeId,
                userId: userId,
                engaged: touchpointId,
                inStore: inStore
            };
        }
        if (_.isUndefined(customerId)) {
            customerId = '';
        }
        if (_.isUndefined(customerId)) {
            inStore = '';
        }
        var promise = new Promise(function (resolve, reject) {
            if (tenantId) {
                payload.tenantId = tenantId;
                _this.api.api('POST', "/v1/customer/engaged/" + customerId, payload)
                    .then(function (response) {
                    var customer = client_1.Customer.fromJson(response);
                    var position = _this.clients.findIndex(function (item) { return item.id === customer.id; });
                    _this.clients.splice(position, 1, customer);
                    _this.clientObservable.next(customer);
                    _this.clientsObservable.next(_this.clients);
                    resolve(customer);
                }, function (response) {
                    reject(response);
                });
            }
            else {
                reject();
            }
        });
        return promise;
    };
    return ClientProvider;
}());
ClientProvider.decorators = [
    { type: core_1.Injectable },
];
ClientProvider.ctorParameters = function () { return [
    { type: http_1.Http, },
    { type: api_provider_1.ApiProvider, },
]; };
exports.ClientProvider = ClientProvider;
//# sourceMappingURL=client.provider.js.map