"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var moment = require("moment");
var Offer = (function () {
    function Offer(offerId, tenantId, customerId, storeId, productId, name, description, discountedPrice, img_url, created) {
        this.offerId = offerId;
        this.tenantId = tenantId;
        this.customerId = customerId;
        this.storeId = storeId;
        this.productId = productId;
        this.name = name;
        this.description = description;
        this.discountedPrice = discountedPrice;
        this.img_url = img_url;
        this.created = created;
        this.discountRate = 0;
        this.product = undefined;
    }
    Offer.createDummyOffer = function () {
        var dummyOffer = new Offer('__DUMMY__', '__DUMMY__', '__DUMMY__', '__DUMMY__', '__DUMMY__', '__DUMMY__', '__DUMMY__', -1, '__DUMMY__', moment('1970-01-01T00:00:00'));
        return dummyOffer;
    };
    Offer.fromJson = function (obj) {
        var _id = (obj.id) ? obj.id : '__FALLBACK__';
        var _tenantId = (obj.tenantId) ? obj.tenantId : '__FALLBACK__';
        var _customerId = (obj.customerId) ? obj.customerId : '__FALLBACK__';
        var _price = (obj.price) ? obj.price : -1;
        var _name = (obj.name) ? obj.name : '__FALLBACK__';
        var _description = (obj.description) ? obj.description : '__FALLBACK__';
        var _img_url = (obj.img_url) ? obj.img_url : '__FALLBACK__';
        var _productId = (obj.productId) ? obj.productId : '__FALLBACK__';
        var _storeId = (obj.storeId) ? obj.storeId : '__FALLBACK__';
        var _created = (obj.created) ? obj.created : moment('1970-01-01T00:00:00');
        return new Offer(_id, _tenantId, _customerId, _storeId, _productId, _name, _description, _price, _img_url, _created);
    };
    Offer.pushOfferIfUnique = function (offerToPush, offers) {
        var foundOffer = _.find(offers, function (o) { return o.getOfferId() === offerToPush.getOfferId(); });
        if (!foundOffer) {
            offers.push(offerToPush);
        }
    };
    Offer.updateOrPushOffer = function (offerToUpdate, offers) {
        var indexOfItemToUpdate = _.findIndex(offers, function (o) {
            return o.getOfferId() === offerToUpdate.getOfferId();
        });
        if (indexOfItemToUpdate !== -1) {
            offers[indexOfItemToUpdate] = offerToUpdate;
        }
        else if (indexOfItemToUpdate === -1) {
            offers.push(offerToUpdate);
            console.warn('Warning: Former version of updated offer was not found in local cache, but cache was synced anyway.');
        }
    };
    Offer.deleteOffer = function (offerToDelete, offers) {
        var indexOfItemToDelete = _.findIndex(offers, function (o) {
            return o.getOfferId() === offerToDelete.getOfferId();
        });
        if (indexOfItemToDelete !== -1) {
            offers.splice(indexOfItemToDelete, 1);
        }
        else if (indexOfItemToDelete === -1) {
            console.warn('Warning: Former version of deleted offer not found in local cache.');
        }
    };
    Offer.prototype.getOfferId = function () { return this.offerId; };
    Offer.prototype.getTenantId = function () { return this.tenantId; };
    Offer.prototype.getCustomerId = function () { return this.customerId; };
    Offer.prototype.getStoreId = function () { return this.storeId; };
    Offer.prototype.getProductName = function () { return this.name; };
    Offer.prototype.getProductDescription = function () { return this.description; };
    Offer.prototype.getProductImgUrl = function () { return this.img_url; };
    Offer.prototype.getCreationDate = function () { return this.created; };
    Offer.prototype.getProduct = function () { return this.product; };
    Offer.prototype.getProductId = function () { return this.productId; };
    Offer.prototype.getDiscountRate = function () { return this.discountRate; };
    Offer.prototype.getDiscountedPrice = function () { return this.discountedPrice; };
    Offer.prototype.setProduct = function (prod) {
        this.product = prod;
        this.discountRate = Math.abs((this.discountedPrice - prod.getPrice()) / (prod.getPrice()));
    };
    return Offer;
}());
exports.Offer = Offer;
//# sourceMappingURL=offer.js.map