"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var Subject_1 = require("rxjs/Subject");
require("rxjs/add/operator/map");
var offer_1 = require("./offer");
var api_provider_1 = require("../api/api.provider");
var _ = require("lodash");
var OffersProvider = (function () {
    function OffersProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this._offers$ = new Subject_1.Subject();
        var dummyOffer = offer_1.Offer.createDummyOffer();
        this.dataStore = { offers: [dummyOffer] };
    }
    Object.defineProperty(OffersProvider.prototype, "offers$", {
        get: function () { return this._offers$.asObservable(); },
        enumerable: true,
        configurable: true
    });
    OffersProvider.prototype.initOffers = function (tenantId, storeId) {
        var _this = this;
        if (this.dataStore.offers[0].getOfferId() !== '__DUMMY__') {
            return new Promise(function (resolve, reject) { resolve(_this.dataStore.offers); });
        }
        else {
            return new Promise(function (resolve, reject) {
                _this.retrieveOffers(tenantId, storeId)
                    .then(function (offers) { resolve(offers); })
                    .catch(function (errors) {
                    console.error("Error on loadOffers() with response: error " + errors[0].code + ",  " + errors[0].msg);
                    reject(errors);
                });
            });
        }
    };
    OffersProvider.prototype.createOffer = function (tenantId, storeId, productId, name, description, discountedPrice, img_url) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId,
            'productId': productId,
            'name': name,
            'description': description,
            'discountedPrice': discountedPrice,
            'img_url': img_url
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/offer', payload)
                .then(function (response) {
                var newOffer = offer_1.Offer.fromJson(response);
                resolve(newOffer);
                if (_this.dataStore.offers[0].getProductName() !== 'DUMMY') {
                    _this.dataStore.offers.splice(0);
                }
                _this.dataStore.offers.push(newOffer);
            })
                .catch(function (error) {
                console.error("Error on createOffer() api request with response: error " + error);
                reject(error);
            });
        });
    };
    OffersProvider.prototype.retrieveOffers = function (tenantId, storeId) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/offers', payload)
                .then(function (response) {
                if (response.length > 0) {
                    _this.dataStore.offers.splice(0);
                    _(response).forEach(function (offer) {
                        _this.dataStore.offers.push(offer_1.Offer.fromJson(offer));
                    });
                    _this._offers$.next(_this.dataStore.offers);
                    resolve(_this.dataStore.offers);
                }
            })
                .catch(function (error_code) {
                if (error_code === 700 || error_code === 400) {
                    resolve([]);
                }
                else {
                    console.error("Error on retrieveOffers() api request with response: error " + error_code);
                    reject(error_code);
                }
            });
        });
    };
    OffersProvider.prototype.retrieveOfferDetails = function (offerId) {
        var _this = this;
        var payload = {};
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/offer/" + offerId, payload)
                .then(function (response) {
                var retrievedOffer = offer_1.Offer.fromJson(response);
                offer_1.Offer.pushOfferIfUnique(retrievedOffer, _this.dataStore.offers);
                _this._offers$.next(_this.dataStore.offers);
                resolve(retrievedOffer);
            })
                .catch(function (errors) {
                console.error("Error on retrieveOfferDetails() api request with response: error " + errors[0].code + ",  " + errors[0].msg);
                reject(errors);
            });
        });
    };
    OffersProvider.prototype.updateOffer = function (offerId, tenantId, storeId, productId, name, description, discountedPrice, img_url) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
            'storeId': storeId,
            'productId': productId,
            'name': name,
            'description': description,
            'price': discountedPrice,
            'img_url': img_url
        };
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/offer/" + offerId, payload)
                .then(function (response) {
                var updatedOffer = offer_1.Offer.fromJson(response);
                offer_1.Offer.updateOrPushOffer(updatedOffer, _this.dataStore.offers);
                _this._offers$.next(_this.dataStore.offers);
                resolve(updatedOffer);
            })
                .catch(function (errors) {
                console.error("Error on updatedOffer() api request with response: error " + errors[0].code + ",  " + errors[0].msg);
                reject(errors);
            });
        });
    };
    OffersProvider.prototype.deleteOffer = function (offerId) {
        var _this = this;
        var payload = {};
        return new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/offer/" + offerId, payload)
                .then(function (response) {
                var deletedOffer = offer_1.Offer.fromJson(response);
                offer_1.Offer.deleteOffer(deletedOffer, _this.dataStore.offers);
                _this._offers$.next(_this.dataStore.offers);
                resolve(deletedOffer);
            })
                .catch(function (errors) {
                console.error("Error on deleteOffer() api request with response: error " + errors[0].code + ",  " + errors[0].msg);
                reject(errors);
            });
        });
    };
    return OffersProvider;
}());
OffersProvider.decorators = [
    { type: core_1.Injectable },
];
OffersProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.OffersProvider = OffersProvider;
//# sourceMappingURL=offers.provider.js.map