"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var product_1 = require("./product");
var api_provider_1 = require("../../services/api/api.provider");
require("rxjs/add/operator/map");
var _ = require("lodash");
var ProductsProvider = (function () {
    function ProductsProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this.data = null;
    }
    ProductsProvider.prototype.create = function (productParameters) {
        var _this = this;
        var payload = productParameters;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', "/v1/product", payload)
                .then(function (response) {
                if (response.id) {
                    resolve(response);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    ProductsProvider.prototype.retrieve = function (tenantId, storeId) {
        var _this = this;
        var payload = {};
        payload.tenantId = tenantId;
        var promise = new Promise(function (resolve, reject) {
            if (_this.data === null) {
                _this.apiProvider.api('GET', "/v1/products/" + storeId, payload)
                    .then(function (response) {
                    _this.data = _this.deserializeProducts(response);
                    resolve(_this.data);
                }, function (err) {
                    reject(err);
                });
            }
            else
                resolve(_this.data);
        });
        return promise;
    };
    ProductsProvider.prototype.deserializeProducts = function (incoming) {
        var products = _.map((_.values(incoming)), product_1.Product.fromJson);
        return products;
    };
    ProductsProvider.prototype.getProduct = function (productId) {
        var product = _.find(this.data, function (obj) { return obj.id === productId; });
        return product;
    };
    ProductsProvider.prototype.retrieveBySKU = function (tenantId, storeId, SKU) {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            var product = _.find(_this.data, function (obj) { return obj.getSKU() === SKU; });
            if (_.isUndefined(product)) {
                var payload = {};
                payload.tenantId = tenantId;
                payload.sku = SKU;
                _this.apiProvider.api('GET', "/v1/products/" + storeId, payload).then(function (response) {
                    if (response.id) {
                        var product_2 = product_1.Product.fromJson(response);
                        _this.data.push(product_2);
                        resolve(product_2);
                    }
                    else {
                        reject();
                    }
                }, function (err) {
                    reject(err);
                });
            }
            else {
                resolve(product);
            }
        });
        return promise;
    };
    return ProductsProvider;
}());
ProductsProvider.decorators = [
    { type: core_1.Injectable },
];
ProductsProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.ProductsProvider = ProductsProvider;
//# sourceMappingURL=products.provider.js.map