"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var barcode_scanner_1 = require("@ionic-native/barcode-scanner");
var qrcode = require("qrcode-generator");
var QRProvider = (function () {
    function QRProvider(barcodeScanner) {
        this.barcodeScanner = barcodeScanner;
    }
    QRProvider.prototype.createQRCode = function (data) {
        try {
            var qr = void 0;
            qr = qrcode(5, 'M');
            qr.addData(data);
            qr.make();
            return qr.createImgTag(4, 2);
        }
        catch (e) {
            console.error('Error generating QR Code:' + e.message);
        }
    };
    QRProvider.prototype.ScanCart = function (userId, cartsProvider) {
        var _this = this;
        var data;
        this.barcodeScanner.scan()
            .then(function (result) {
            if (!result.cancelled) {
                data = result.text;
                _this.parseQRCode(data, userId, cartsProvider);
                console.log(data);
            }
        })
            .catch(function (err) {
            alert(err);
        });
    };
    QRProvider.prototype.parseQRCode = function (scannedData, userId, cartsProvider) {
        var _this = this;
        var qrCodeValues = scannedData.split('?');
        var cartId = qrCodeValues[0];
        cartsProvider.retrieve(cartId).then(function (cartToPass) {
            cartsProvider.retrieveAllCarts().then(function (response) {
                cartsProvider.retrieveByUserAndStore(cartToPass.tenantId, cartToPass.storeId, userId).then(function (cartsOfUser) {
                    cartsOfUser = cartsOfUser.filter(function (cart) { return cart.status === 'open'; });
                    if (cartsOfUser.length > 0) {
                        _this.passCartItems(cartToPass, cartsOfUser[0], cartsProvider);
                    }
                    else {
                        cartsProvider.create(cartToPass.tenantId, cartToPass.getStoreId(), userId).then(function (destiNationCart) {
                            _this.passCartItems(cartToPass, destiNationCart, cartsProvider);
                        });
                    }
                });
            });
        });
    };
    QRProvider.prototype.passCartItems = function (source, destination, cartsProvider) {
        source.getCartItems().forEach(function (cItem) {
            cartsProvider.addItem(destination.id, cItem);
        });
    };
    return QRProvider;
}());
QRProvider.decorators = [
    { type: core_1.Injectable },
];
QRProvider.ctorParameters = function () { return [
    { type: barcode_scanner_1.BarcodeScanner, },
]; };
exports.QRProvider = QRProvider;
//# sourceMappingURL=qrCode.provider.js.map