"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var api_provider_1 = require("../../services/api/api.provider");
var touchpoint_1 = require("./touchpoint");
require("rxjs/add/operator/map");
var Subject_1 = require("rxjs/Subject");
var _ = require("lodash");
var TouchpointProvider = (function () {
    function TouchpointProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this.touchpoints = [];
        this.touchpointObservable = new Subject_1.Subject();
        this.touchpointsObservable = new Subject_1.Subject();
    }
    TouchpointProvider.prototype.create = function (touchpointParameters) {
        var _this = this;
        var payload = touchpointParameters;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/touchpoint', payload)
                .then(function (response) {
                if (response.id) {
                    _this.currentTouchpoint = touchpoint_1.Touchpoint.fromJson(response);
                    _this.touchpoints.push(_this.currentTouchpoint);
                    _this.touchpointState(_this.currentTouchpoint);
                    resolve(_this.currentTouchpoint);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.retrieveDetails = function (tenantId, touchpointId) {
        var _this = this;
        var payload = {};
        payload.tenantId = tenantId;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/touchpoint/" + touchpointId, payload)
                .then(function (response) {
                if (response.id) {
                    var touchpoint_2 = touchpoint_1.Touchpoint.fromJson(response);
                    var position = _this.touchpoints.findIndex(function (item) { return item.id === touchpoint_2.id; });
                    _this.touchpoints.splice(position, 1, touchpoint_2);
                    _this.touchpointState(touchpoint_2);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.retrieveTouchpoints = function (tenantId, storeId) {
        var _this = this;
        var payload = {};
        if (tenantId)
            payload.tenantId = tenantId;
        if (storeId)
            payload.storeId = storeId;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/touchpoints', payload)
                .then(function (response) {
                _this.touchpoints = _.map((_.values(response)), touchpoint_1.Touchpoint.fromJson);
                _this.touchpointsObservable.next(_this.touchpoints);
                resolve(_this.touchpoints);
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.update = function (touchpointId, obj) {
        var _this = this;
        var payload = obj;
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', '/v1/touchpoint/' + touchpointId, obj)
                .then(function (response) {
                if (response.id) {
                    var touchpoint_3 = touchpoint_1.Touchpoint.fromJson(response);
                    var position = _this.touchpoints.findIndex(function (item) { return item.id === touchpoint_3.id; });
                    _this.touchpoints.splice(position, 1, touchpoint_3);
                    _this.touchpointState(touchpoint_3);
                    resolve(_this.currentTouchpoint);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.delete = function (touchpointId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', '/v1/touchpoint/' + touchpointId, payload)
                .then(function (response) {
                if (response.id) {
                    var touchpoint_4 = touchpoint_1.Touchpoint.fromJson(response);
                    var position = _this.touchpoints.findIndex(function (item) { return item.id === touchpoint_4.id; });
                    _this.touchpoints.splice(position, 1);
                    _this.touchpointState(touchpoint_4);
                    resolve(touchpoint_4);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.addUser = function (touchpointId, userId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/touchpoint/" + touchpointId + "/user/" + userId, payload)
                .then(function (response) {
                if (response.id) {
                    var touchpoint_5 = touchpoint_1.Touchpoint.fromJson(response);
                    var position = _this.touchpoints.findIndex(function (item) { return item.id === touchpoint_5.id; });
                    _this.touchpoints.splice(position, 1, touchpoint_5);
                    _this.touchpointObservable.next(touchpoint_5);
                    _this.touchpointsObservable.next(_this.touchpoints);
                    resolve(touchpoint_5);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.removeUser = function (touchpointId, userId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/touchpoint/" + touchpointId + "/user/" + userId, payload)
                .then(function (response) {
                if (response.id) {
                    var touchpoint_6 = touchpoint_1.Touchpoint.fromJson(response);
                    var position = _this.touchpoints.findIndex(function (item) { return item.id === touchpoint_6.id; });
                    _this.touchpoints.splice(position, 1, touchpoint_6);
                    _this.touchpointObservable.next(touchpoint_6);
                    _this.touchpointsObservable.next(_this.touchpoints);
                    resolve(touchpoint_6);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    TouchpointProvider.prototype.touchpointState = function (result) {
        this.currentTouchpoint = result;
        this.touchpointObservable.next(this.currentTouchpoint);
        this.touchpointsObservable.next(this.touchpoints);
    };
    return TouchpointProvider;
}());
TouchpointProvider.decorators = [
    { type: core_1.Injectable },
];
TouchpointProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.TouchpointProvider = TouchpointProvider;
//# sourceMappingURL=touchpoint.provider.js.map