"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var cart_1 = require("../cart/cart");
var WishList = (function () {
    function WishList(tenantId, wishItems, type, name, userId, id) {
        this.tenantId = tenantId;
        this.wishItems = wishItems;
        this.type = type;
        this.name = name;
        this.userId = userId;
        this.id = id;
        this.calculateWishlistCost();
        if (this.id === undefined) {
            this.id = '';
        }
    }
    WishList.fromJson = function (obj) {
        return new WishList(obj.tenantId, _.map((_.values(obj.items)), WishItem.fromJson), obj.type, obj.name, obj.userId, obj.id);
    };
    WishList.prototype.toJson = function () {
        var wishlistJson = {
            id: this.id,
            userId: this.userId,
            name: this.name,
            tenantId: '',
            __v: '',
            created: '',
            numItems: this.getItemsCount(),
            items: [],
            type: this.type
        };
        return wishlistJson;
    };
    WishList.prototype.calculateWishlistCost = function () {
        var _this = this;
        this.wishCost = 0;
        this.wishItems.forEach(function (cItem) {
            _this.wishCost += cItem.getCost();
        });
    };
    WishList.prototype.findItemInWishlistItems = function (keyItem) {
        return _.find(this.wishItems, function (cItem) {
            return cItem.getProduct().getId() === keyItem.getProduct().getId();
        });
    };
    WishList.prototype.findProductInWishlistItems = function (keyId) {
        return _.find(this.wishItems, function (cItem) {
            return cItem.getProduct().getId() === keyId;
        });
    };
    WishList.prototype.deleteWishlist = function () {
    };
    WishList.prototype.addWishlistItem = function (newItem) {
        var foundItem = this.findItemInWishlistItems(newItem);
        if (foundItem) {
            foundItem.updateItemQuantity(foundItem.getQuantity() + newItem.getQuantity());
        }
        else {
            this.wishItems.push(newItem);
        }
        this.calculateWishlistCost();
    };
    WishList.prototype.removeWishlistItem = function (idOfProductToRemove) {
        var indexOfItemToRemove = _.findIndex(this.wishItems, function (cItem) {
            return cItem.getProduct().getId() === idOfProductToRemove;
        });
        if (indexOfItemToRemove === -1) {
            console.error('Failed to remove product with id "'
                + idOfProductToRemove + '" from cart of "');
        }
        else {
            this.wishItems.splice(indexOfItemToRemove, 1);
            this.calculateWishlistCost();
        }
    };
    WishList.prototype.getItem = function (prodId) {
        var indexOfItem = _.findIndex(this.wishItems, function (cItem) {
            return cItem.getProduct().getId() === prodId;
        });
        return this.wishItems[indexOfItem];
    };
    WishList.prototype.incrementQuantity = function (prodId) {
        var foundItem = this.findProductInWishlistItems(prodId);
        foundItem.updateItemQuantity(foundItem.getQuantity() + 1);
        this.calculateWishlistCost();
    };
    WishList.prototype.decrementQuantity = function (prodId) {
        var foundItem = this.findProductInWishlistItems(prodId);
        foundItem.updateItemQuantity(foundItem.getQuantity() - 1);
        this.calculateWishlistCost();
    };
    WishList.prototype.getWishlistCost = function () { return this.wishCost; };
    WishList.prototype.getItemsCount = function () { return this.wishItems.length; };
    WishList.prototype.getTenantId = function () { return this.tenantId; };
    WishList.prototype.getWishlistItems = function () { return this.wishItems; };
    return WishList;
}());
exports.WishList = WishList;
var WishItem = (function () {
    function WishItem(cartItem, storeId) {
        this.cartItem = cartItem;
        this.storeId = storeId;
        this.cost = this.getProduct().getPrice() * this.getQuantity();
    }
    WishItem.fromJson = function (obj) {
        return new WishItem(cart_1.CartItem.fromJson(obj), obj.storeId);
    };
    WishItem.prototype.toJson = function () {
        var wishItem = this.cartItem.toJson();
        wishItem.storeId = this.storeId;
        return wishItem;
    };
    WishItem.prototype.updateItemQuantity = function (updatedQuantity) {
        this.getCartItem().quantity = updatedQuantity;
        this.cost = this.getCartItem().product.getPrice() * this.getQuantity();
    };
    WishItem.prototype.getCost = function () { return this.cost; };
    WishItem.prototype.getProduct = function () { return this.getCartItem().getProduct(); };
    WishItem.prototype.getQuantity = function () { return this.getCartItem().getQuantity(); };
    WishItem.prototype.getSize = function () { return this.getCartItem().getSize(); };
    WishItem.prototype.getColour = function () { return this.getCartItem().getColour(); };
    WishItem.prototype.getStoreId = function () { return this.storeId; };
    WishItem.prototype.getCartItem = function () { return this.cartItem; };
    return WishItem;
}());
exports.WishItem = WishItem;
//# sourceMappingURL=wishlist.js.map