"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var api_provider_1 = require("../api/api.provider");
var cart_1 = require("../cart/cart");
var Subject_1 = require("rxjs/Subject");
var wishlist_1 = require("./wishlist");
require("rxjs/add/operator/map");
var _ = require("lodash");
var product_1 = require("../product/product");
var WishListsProvider = (function () {
    function WishListsProvider(apiProvider) {
        this.apiProvider = apiProvider;
        this.wishlists = [];
        this.wishlists = [];
        this.currentWishlist = wishlist_1.WishList.fromJson(this.emptyWishlist());
        this.wishListObservable = new Subject_1.Subject();
        this.wishListsObservable = new Subject_1.Subject();
    }
    WishListsProvider.prototype.wishlistState = function (result) {
        this.currentWishlist = result;
        this.wishListObservable.next(this.currentWishlist);
        this.wishListsObservable.next(this.wishlists);
    };
    WishListsProvider.prototype.emptyWishlist = function (tenantId, id, name, user, type) {
        var empty;
        empty = [];
        return {
            'id': id,
            'userId': user,
            'name': name,
            'tenantId': tenantId,
            '__v': '',
            'created': '',
            'items': empty,
            'numItems': 0,
            'type': type
        };
    };
    WishListsProvider.prototype.create = function (tenantId, name, type) {
        var _this = this;
        var payload = {};
        payload = {
            name: name,
            type: type,
            tenantId: tenantId
        };
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', '/v1/list', payload)
                .then(function (response) {
                if (response.id) {
                    var list = wishlist_1.WishList.fromJson(response);
                    _this.wishlists.push(list);
                    _this.wishlistState(list);
                    console.log('resolving');
                    resolve(list);
                }
                else {
                    reject('Error wish#1');
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.updateList = function (wishlist, userId) {
        var _this = this;
        var payload = {
            type: wishlist.type,
            name: userId,
            tenantId: wishlist.tenantId
        };
        console.log('updating...');
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/list/" + wishlist.id, payload)
                .then(function (response) {
                if (response.id) {
                    var list_1 = wishlist_1.WishList.fromJson(response);
                    var position = _this.wishlists.findIndex(function (item) { return item.id === list_1.id; });
                    _this.wishlists.splice(position, 1, list_1);
                    _this.wishlistState(list_1);
                    resolve(list_1);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.delete = function (cartId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/list/" + cartId, payload)
                .then(function (response) {
                if (response.id) {
                    var list_2 = wishlist_1.WishList.fromJson(response);
                    var position = _this.wishlists.findIndex(function (item) { return item.id === list_2.id; });
                    _this.wishlists.splice(position, 1);
                    _this.wishListsObservable.next(_this.wishlists);
                    resolve(response);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.addItem = function (cartId, product) {
        var _this = this;
        var payload = {};
        payload = {
            productId: product.getProduct().id,
            quantity: product.getQuantity(),
            finalPrice: product.getCost(),
            title: product.getProduct().getName(),
            storeId: product.getStoreId()
        };
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('POST', "/v1/list/" + cartId, payload)
                .then(function (response) {
                if (response.id) {
                    var list_3 = wishlist_1.WishList.fromJson(response);
                    var position = _this.wishlists.findIndex(function (item) { return item.id === list_3.id; });
                    _this.wishlists.splice(position, 1, list_3);
                    _this.wishlistState(list_3);
                    resolve(list_3);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.deleteItem = function (cartId, product, batch) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('DELETE', "/v1/list/" + cartId + "/" + product.getProduct().id, payload)
                .then(function (response) {
                if (response.id) {
                    var list_4 = wishlist_1.WishList.fromJson(response);
                    if (batch !== true) {
                        var position = _this.wishlists.findIndex(function (item) { return item.id === list_4.id; });
                        _this.wishlists.splice(position, 1, list_4);
                        _this.wishlistState(list_4);
                    }
                    resolve(list_4);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.emptyList = function (id) {
        var _this = this;
        var wish = _.find(this.wishlists, function (o) { return o.id === id; });
        var promiseStack = [];
        wish.getWishlistItems().forEach(function (cItem) {
            promiseStack.push(_this.deleteItem(id, cItem, true));
        });
        Promise.all(promiseStack).then(function () {
            console.log('all ok');
        });
    };
    WishListsProvider.prototype.retrieveByUser = function (tenantId, userId) {
        var _this = this;
        var payload = {
            'tenantId': tenantId,
        };
        var promise = new Promise(function (resolve, reject) {
            if (_this.wishlists.length > 0 && !_.isUndefined(_this.wishlists[0].name))
                resolve(_this.wishlists);
            else {
                _this.apiProvider.api('GET', '/v1/lists', payload)
                    .then(function (response) {
                    if (response) {
                        var lists_1 = [];
                        response.forEach(function (item) {
                            lists_1.push(wishlist_1.WishList.fromJson(item));
                        });
                        _this.wishlists = lists_1.filter(function (item) { return (item.name === userId && item.getTenantId() === tenantId); });
                        _this.wishListsObservable.next(_this.wishlists);
                        console.log('resolving');
                        resolve(_this.wishlists);
                    }
                    else {
                        reject();
                    }
                }, function (err) {
                    reject(err);
                });
            }
        });
        return promise;
    };
    WishListsProvider.prototype.retrieve = function () {
        var _this = this;
        var payload = {};
        payload.userId = 'randomID';
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/lists', payload)
                .then(function (response) {
                if (response) {
                    var lists_2 = [];
                    response.forEach(function (item) {
                        lists_2.push(wishlist_1.WishList.fromJson(item));
                    });
                    _this.wishlists = lists_2;
                    _this.wishListsObservable.next(_this.wishlists);
                    console.log('resolving');
                    resolve(lists_2);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.retrieveListDetails = function (cartId, tenantId) {
        var _this = this;
        var payload = {
            'tenantId': tenantId
        };
        var promise = new Promise(function (resolve, reject) {
            var wishlist_temp = _this.wishlists.filter(function (item) { return cartId === item.id; });
            if (wishlist_temp.length > 0)
                resolve(wishlist_temp[0]);
            else {
                _this.apiProvider.api('GET', "/v1/list" + cartId, payload)
                    .then(function (response) {
                    if (response.id) {
                        console.log('resolving');
                        var list = wishlist_1.WishList.fromJson(response);
                        _this.currentWishlist = list;
                        _this.wishListObservable.next(_this.currentWishlist);
                        resolve(list);
                    }
                    else {
                        reject();
                    }
                }, function (err) {
                    reject(err);
                });
            }
        });
        return promise;
    };
    WishListsProvider.prototype.retrieveListItem = function (cartId, productId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', "/v1/list/" + cartId + "/" + productId, payload)
                .then(function (response) {
                if (response) {
                    console.log('resolving');
                    var product = product_1.Product.fromJson(response);
                    resolve(product);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.updateListItem = function (cartId, cItem) {
        var _this = this;
        var payload = {
            productId: cItem.getProduct().id,
            quantity: cItem.getQuantity(),
            finalPrice: cItem.getCost(),
            title: cItem.getProduct().getName(),
            storeId: cItem.getStoreId()
        };
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('PUT', "/v1/list/" + cartId + "/" + cItem.getProduct().id, payload)
                .then(function (response) {
                if (response) {
                    console.log('resolving');
                    resolve(response);
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.retrieveDez = function (cartId) {
        var _this = this;
        var payload = {};
        var promise = new Promise(function (resolve, reject) {
            _this.apiProvider.api('GET', '/v1/cart/' + cartId, payload)
                .then(function (response) {
                if (response._id) {
                    _this.wishlists[response.id] = response;
                    _this.currentWishlist = response;
                    console.log('resolving');
                    resolve(_this.deserializeWishlist(_this.currentWishlist));
                }
                else {
                    reject();
                }
            }, function (err) {
                reject(err);
            });
        });
        return promise;
    };
    WishListsProvider.prototype.deserializeWishlist = function (incoming) {
        var items;
        items = [];
        for (var _i = 0, _a = incoming.items; _i < _a.length; _i++) {
            var item = _a[_i];
            var tempCartItem = new wishlist_1.WishItem(cart_1.CartItem.fromJson(item), item.storeId);
            items.push(tempCartItem);
        }
        return new wishlist_1.WishList(undefined, items, incoming.type, incoming.name, incoming._id);
    };
    WishListsProvider.prototype.deserializeWishlists = function (incoming) {
        var wishlistsN = _.map((_.values(incoming)), wishlist_1.WishList.fromJson);
        return wishlistsN;
    };
    return WishListsProvider;
}());
WishListsProvider.decorators = [
    { type: core_1.Injectable },
];
WishListsProvider.ctorParameters = function () { return [
    { type: api_provider_1.ApiProvider, },
]; };
exports.WishListsProvider = WishListsProvider;
//# sourceMappingURL=wishlist.provider.js.map